local Graph = AceLibrary("Graph-1.0")
local me={}

local GraphColors={
	Damage={1.0,0.0,0.0,1.0},
	DamageTaken={1.0,1.0,0.0,1.0},
	Healing={0.0,1.0,0.0,1.0},
	HealingTaken={0.0,1.0,1.0,1.0},
	Overhealing={0.5,0.0,1.0,1.0},
}

local GraphName={
	Damage="Damage",
	DamageTaken="Damage Taken",
	Healing="Healing",
	HealingTaken="Healing Taken",
	Overhealing="Overhealing",
}

function me:RefreshGraph()
	local Graph=Recount.GraphWindow.LineGraph
	Graph:ResetData()

	local i=1
	for k,v in pairs(Recount.GraphWindow.Data) do		
		if Recount.GraphWindow.Rows[i].Enabled:GetChecked() then
			local color=GraphColors[k]
			if type(color)~="table" then
				local r,g,b,a=Recount.GraphWindow.Rows[i].Key:GetVertexColor()
				color={r,g,b,a}
			end
			Graph:AddDataSeries(v,color)
		end
		i=i+1
	end
end

function Recount:SetGraphData(name,data,combat)
	Recount.GraphWindow.Title:SetText("Graph Window - "..name)
	Recount.GraphWindow.Data=data
	Recount.GraphWindow.CombatTime=combat
	Recount.GraphWindow:Show()
	
	local i=1
	for k, _ in pairs(Recount.GraphWindow.Data) do
		local color=GraphColors[k]

		if color==nil then
			color={math.random(),math.random(),math.random(),1.0}
		end

		if i<=10 then
			Recount.GraphWindow.Rows[i]:Show()
			Recount.GraphWindow.Rows[i].Key:SetVertexColor(color[1],color[2],color[3],1)
			Recount.GraphWindow.Rows[i].Name:SetText(GraphName[k] or k)
		end

		i=i+1
	end

	for j=i,10 do
		Recount.GraphWindow.Rows[j]:Hide()
	end

	for i=1,10 do 
		local c=combat[i]
		if c then
			Recount.GraphWindow.TimeRows[i]:Show()
			--Recount.GraphWindow.TimeRows[i]
			Recount.GraphWindow.TimeRows[i].Start:SetText(c[3])
			Recount.GraphWindow.TimeRows[i].End:SetText(c[4])
		else
			Recount.GraphWindow.TimeRows[i]:Hide()
		end
	end

	me:RefreshGraph()
end

function Recount:CreateGraphWindow()
	Recount.GraphWindow=CreateFrame("Frame","Recount_GraphWindow",UIParent)

	local theFrame=Recount.GraphWindow

	theFrame:ClearAllPoints()
	theFrame:SetPoint("CENTER",UIParent)
	theFrame:SetHeight(432)
	theFrame:SetWidth(600)

	theFrame:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		edgeFile = "Interface\\AddOns\\Recount\\textures\\otravi-semi-full-border", edgeSize = 32,
		insets = {left = 1, right = 1, top = 20, bottom = 1},
	})
	theFrame:SetBackdropBorderColor(1.0,0.0,0.0)
	theFrame:SetBackdropColor(24/255, 24/255, 24/255)

	theFrame:EnableMouse(true)
	theFrame:SetMovable(true)

	theFrame:SetScript("OnMouseDown", function() 
						if ( ( ( not this.isLocked ) or ( this.isLocked == 0 ) ) and ( arg1 == "LeftButton" ) ) then
						  this:StartMoving();
						  this.isMoving = true;
						 end
						end)
	theFrame:SetScript("OnMouseUp", function() 
						if ( this.isMoving ) then
						  this:StopMovingOrSizing();
						  this.isMoving = false;
						 end
						end)
					
	theFrame:SetScript("OnHide", function()
						if ( this.isMoving ) then
						  this:StopMovingOrSizing();
						  this.isMoving = false;
						 end
						end)

	theFrame.Title=theFrame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	theFrame.Title:SetPoint("TOPLEFT",theFrame,"TOPLEFT",6,-15)
	theFrame.Title:SetTextColor(1.0,1.0,1.0)
	theFrame.Title:SetText("Graph Window")

	theFrame.LineGraph=Graph:CreateGraphLine("Recount_GraphWindow_LineGraph",theFrame,"LEFT","LEFT",1,-16,400,400)
	theFrame.LineGraph:SetAutoScale(true)
	theFrame.LineGraph:SetYAxis(0,100)
	theFrame.LineGraph:SetGridSpacing(15,25)
	theFrame.LineGraph:LockYMin(true)


	theFrame.Labels=CreateFrame("Frame",nil,theFrame)
	local Labels=theFrame.Labels

	Labels:SetWidth(194)
	Labels:SetHeight(16)
	Labels:SetPoint("TOPRIGHT",theFrame,"TOPRIGHT",-1,-34)

	Labels.Key=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Key:SetPoint("LEFT",Labels,"LEFT",1,0)
	Labels.Key:SetText("K")

	Labels.Name=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Name:SetPoint("LEFT",Labels,"LEFT",16,0)
	Labels.Name:SetText("Data Name")

	Labels.Name=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Name:SetPoint("Right",Labels,"RIGHT",-2,0)
	Labels.Name:SetText("Enabled")

	theFrame.Rows={}

	for i=1,10 do
		local Row=CreateFrame("Frame",nil,theFrame)
		Row:SetWidth(194)
		Row:SetHeight(16)
		Row:SetPoint("TOPRIGHT",theFrame,"TOPRIGHT",-1,-34-i*18)

		Row.Key=Row:CreateTexture(nil,"OVERLAY")
		Row.Key:SetPoint("LEFT",Row,"LEFT",0,0)
		Row.Key:SetTexture("Interface\\Buttons\\WHITE8X8.blp")
		Row.Key:SetWidth(12)
		Row.Key:SetHeight(12)

		Row.Name=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.Name:SetPoint("LEFT",Row,"LEFT",16,0)
		Row.Name:SetText("Test")
		Row.Name:SetTextColor(1.0,1.0,1.0,1.0)
		
		Row.Enabled=CreateFrame("CheckButton",nil,Row)
		Row.Enabled:SetPoint("RIGHT",Row,"RIGHT",-4,0)
		Row.Enabled:SetWidth(16)
		Row.Enabled:SetHeight(16)
		Row.Enabled:SetScript("OnClick",function () if this:GetChecked() then this:SetChecked(true) else this:SetChecked(false) end me:RefreshGraph() end)
		Row.Enabled:SetNormalTexture("Interface\\Buttons\\UI-CheckBox-Up")
		Row.Enabled:SetPushedTexture("Interface\\Buttons\\UI-CheckBox-Down")
		Row.Enabled:SetCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check")
		Row.Enabled:SetHighlightTexture("Interface\\Buttons\\UI-CheckBox-Highlight")
		Row.Enabled:SetChecked(true)

		Row:Hide()

		table.insert(theFrame.Rows,Row)
	end

	theFrame.TimeLabels=CreateFrame("Frame",nil,theFrame)
	local Labels=theFrame.TimeLabels

	Labels:SetWidth(180)
	Labels:SetHeight(16)
	Labels:SetPoint("TOPLEFT",theFrame.Rows[10],"BOTTOMLEFT",0,-2)

	Labels.Who=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Who:SetPoint("LEFT",Labels,"LEFT",0,0)
	Labels.Who:SetText("Fought")

	Labels.Start=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.Start:SetPoint("CENTER",Labels,"CENTER",5,0)
	Labels.Start:SetText("Start")

	Labels.End=Labels:CreateFontString(nil,"OVERLAY","GameFontNormal")
	Labels.End:SetPoint("RIGHT",Labels,"RIGHT",-2,0)
	Labels.End:SetText("End")

	theFrame.TimeRows={}
	for i=1,10 do
		local Row=CreateFrame("Frame",nil,theFrame)
		Row:SetWidth(194)
		Row:SetHeight(16)
		Row:SetPoint("TOP",Labels,"BOTTOM",0,-2-18*(i-1))

		Row.Who=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.Who:SetPoint("LEFT",Row,"LEFT",0,0)
		Row.Who:SetText("Test")
		Row.Who:SetTextColor(1.0,1.0,1.0,1.0)

		Row.Start=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.Start:SetPoint("RIGHT",Row,"RIGHT",-65,0)
		Row.Start:SetText("12:12:50")
		Row.Start:SetTextColor(1.0,1.0,1.0,1.0)


		Row.End=Row:CreateFontString(nil,"OVERLAY","GameFontNormal")
		Row.End:SetPoint("RIGHT",Row,"RIGHT",0,0)
		Row.End:SetText("12:12:50")
		Row.End:SetTextColor(1.0,1.0,1.0,1.0)

		--Row:Hide()

		table.insert(theFrame.TimeRows,Row)
	end

	theFrame.CloseButton=CreateFrame("Button",nil,theFrame)
	theFrame.CloseButton:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up.blp")
	theFrame.CloseButton:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down.blp")
	theFrame.CloseButton:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight.blp")
	theFrame.CloseButton:SetWidth(20)
	theFrame.CloseButton:SetHeight(20)
	theFrame.CloseButton:SetPoint("TOPRIGHT",theFrame,"TOPRIGHT",-4,-12)
	theFrame.CloseButton:SetScript("OnClick",function() this:GetParent():Hide() end)

	theFrame:Hide()
end


