Recount = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0","AceDB-2.0","AceEvent-2.0")
Recount:RegisterDB("RecountDB","RecountPerCharDB")

local Roster = AceLibrary("Roster-2.1")
local Parser = ParserLib:GetInstance("1.1");

local DataVersion = 3

Recount.consoleOptions = {
	type = 'group',
	args = {
		reset = {
			name = "Reset",
			desc = "Resets the data",
			type = 'execute',
			func = function() Recount.db.char.combatants={} end
		},
		show = {
			name = "Show",
			desc = "Shows the main window",
			type = 'execute',
			func = function() Recount.MainWindow:Show() end
		}
	}
}

Recount:RegisterChatCommand({"/recount"}, Recount.consoleOptions)

function Recount:FindUnit(name)
	for unit in Roster:IterateRoster(true) do
		if unit.name==name then
			return unit.unitid
		elseif UnitName(unit.unitid.."target")==name then
			return unit.unitid.."target"
		end
	end
	return nil
end

function Recount:InitFightData(data)
	--Init Data tracked
	data.Damage=0
	data.FDamage=0
	data.DamageTaken=0
	data.Healing=0
	data.HealingTaken=0
	data.Overhealing=0

	--Ability Data
	data.Attacks={}
	data.FAttacks={}
	data.Heals={}
	data.OverHeals={}	

	--Interaction Data
	data.DamagedWho={} --Who did I damage?
	data.FDamagedWho={} --Who did I damage?
	data.WhoDamaged={} --Who damaged me?
	data.HealedWho={} --Who did I heal?
	data.WhoHealed={} --Who healed me?

	--Elemental Tracking
	data.ElementDone={}
	data.ElementDoneResist={}
	data.ElementTaken={}
	data.ElementTakenResist={}
end

function Recount:AddCombatant(name)
	Recount.db.char.combatants[name]={}

	local combatant=Recount.db.char.combatants[name]

	local unit=Recount:FindUnit(name)
	if unit then
		combatant.isFriend=UnitIsFriend("player",unit)
		combatant.class, combatant.enClass=UnitClass(unit)
		combatant.unit=unit
	else
		combatant.isFriend=false
		combatant.enClass="WARRIOR"
		combatant.checkLater=true		
	end
	
	combatant.OverallData={}
	combatant.CurrentFightData={}
	combatant.LastFightData={}

	Recount:InitFightData(combatant.OverallData)
	Recount:InitFightData(combatant.CurrentFightData)
	Recount:InitFightData(combatant.LastFightData)
	

	combatant.TimeWindows={}
	local TimeWindows=combatant.TimeWindows
	
	TimeWindows.Damage={}
	TimeWindows.DamageTaken={}
	TimeWindows.Healing={}
	TimeWindows.Overhealing={}
	TimeWindows.HealingTaken={}

	for i=1,25 do
		TimeWindows.Damage[i]=0
		TimeWindows.DamageTaken[i]=0
		TimeWindows.Healing[i]=0
		TimeWindows.Overhealing[i]=0
		TimeWindows.HealingTaken[i]=0
	end

	combatant.TimeData={}
	local TimeData=combatant.TimeData

	TimeData.Damage={}
	TimeData.DamageTaken={}
	TimeData.Healing={}
	TimeData.Overhealing={}
	TimeData.HealingTaken={}

	combatant.TimeNeedZero={}
end

function Recount:RecheckCombatants()
	for k,v in Recount.db.char.combatants do
		
	end
end



--.MainTitle = What you see on the window title 
--.TopNames = Names of the entries for the top data
--.TopAmount = What we call the amount for the top
--.BotNames = Names of the entries for the bottom
--.BotMin = The minimum label for bottom
--.BotAvg = The average label for bottom
--.BotMax = The minimum label for bottom
--.BotAmount = Label for what the amount is on the bottom
local DetailTitles={}
DetailTitles.Attacks={
	TopNames = "Ability Name",
	TopAmount = "Damage",
	BotNames = "Type",
	BotMin = "Min",
	BotAvg = "Avg",
	BotMax = "Max",
	BotAmount = "Count"
}

DetailTitles.DamagedWho={
	TopNames = "Player/Mob Name",
	TopAmount = "Damage",
	BotNames = "Attack Name",
	BotMin = "",
	BotAvg = "",
	BotMax = "",
	BotAmount = "Damage"
}

DetailTitles.Heals={
	TopNames = "Heal Name",
	TopAmount = "Heal",
	BotNames = "Type",
	BotMin = "Min",
	BotAvg = "Avg",
	BotMax = "Max",
	BotAmount = "Count"
}

DetailTitles.HealedWho={
	TopNames = "Player/Mob Name",
	TopAmount = "Healed",
	BotNames = "Heal Name",
	BotMin = "",
	BotAvg = "",
	BotMax = "",
	BotAmount = "Healed"
}

DetailTitles.OverHeals={
	TopNames = "Heal Name",
	TopAmount = "Overheal",
	BotNames = "Type",
	BotMin = "Min",
	BotAvg = "Avg",
	BotMax = "Max",
	BotAmount = "Count"
}

local DataModes={}

function DataModes:DamageReturner(data)
	return (data.OverallData.Damage or 0), {{data.OverallData.Attacks,"'s Hostile Attacks",DetailTitles.Attacks},{data.OverallData.DamagedWho," Damaged Who",DetailTitles.DamagedWho}}
end

function DataModes:FriendlyDamageReturner(data)
	return (data.OverallData.FDamage or 0), {{data.OverallData.FAttacks,"'s Friendly Fire",DetailTitles.Attacks},{data.OverallData.FDamagedWho," Friendly Fired On",DetailTitles.DamagedWho}}
end

function DataModes:DamageTakenReturner(data)
	return (data.OverallData.DamageTaken or 0), {{data.OverallData.WhoDamaged," Took Damage From",DetailTitles.DamagedWho}}
end

function DataModes:HealingReturner(data)
	return (data.OverallData.Healing or 0), {{data.OverallData.Heals,"'s Effective Healing",DetailTitles.Heals},{data.OverallData.HealedWho," Healed Who",DetailTitles.HealedWho},{data.OverallData.OverHeals,"'s Overhealing",DetailTitles.OverHeals}}
end

function DataModes:HealingTaken(data)
	return (data.OverallData.HealingTaken or 0), {{data.OverallData.WhoHealed," was Healed by",DetailTitles.HealedWho}}
end

function DataModes:OverhealingReturner(data)
	return (data.OverallData.Overhealing or 0), {{data.OverallData.OverHeals,"'s Overhealing",DetailTitles.OverHeals},{data.OverallData.Heals,"'s Effective Healing",DetailTitles.Heals},{data.OverallData.HealedWho," Healed Who",DetailTitles.HealedWho}}
end

local MainWindowModes={
{"Damage Done",DataModes.DamageReturner},
{"Friendly Fire",DataModes.FriendlyDamageReturner},
{"Damage Taken",DataModes.DamageTakenReturner},
{"Healing Done",DataModes.HealingReturner},
{"Healing Taken",DataModes.HealingTaken},
{"Overhealing Done",DataModes.OverhealingReturner}}

function Recount:RosterChange()
	for k,v in Recount.db.char.combatants do
		local unitid = Roster:GetUnitIDFromName(k)

		--Must be in our group
		if unitid then
			
		end
	end
end

local TimeWeight = 1/25
function Recount:TimeTick()
	local Time=GetTime()
	local TimeFormatted

	--First check if combat status changed
	Recount:CheckCombat(Time)

	--Need to increment where data gets put and erase the old ones
	Recount.TimeStep=Recount.TimeStep+1
	if Recount.TimeStep>25 then
		Recount.TimeStep=1
	end

	for name,v in pairs(Recount.db.char.combatants) do		
		for k, v2 in pairs(v.TimeWindows) do
			local NewEntry=0

			for _,v3 in pairs(v2) do
				NewEntry=NewEntry+v3
			end
			
			if NewEntry>0 then
				--do we need a leading zero?
				if not v.TimeNeedZero[k] then
					table.insert(v.TimeData[k],{Time-1, 0})
					v.TimeNeedZero[k]=true
				end
				NewEntry=NewEntry*TimeWeight
				table.insert(v.TimeData[k],{Time, NewEntry})
			elseif v.TimeNeedZero[k] then --Check if we need a trailing zero
				table.insert(v.TimeData[k],{Time, 0})
				v.TimeNeedZero[k]=false
			end
			--Need to set where we will be putting new data to 0
			v2[Recount.TimeStep]=0
		end

		--Lets see if this unit needs to be updated		
		if v.checkLater then
			local Unit=Recount:FindUnit(name)
			if Unit then
				v.isFriend=UnitIsFriend("player",unit)
				v.class, v.enClass=UnitClass(unit)
				v.checkLater=false
			end
		end		
	end
end

function Recount:CheckCombat(Time)
	local TimeFormatted
	for unit in Roster:IterateRoster(true) do
		if UnitAffectingCombat(unit.unitid) then
			if not Recount.InCombat then	
				local Hour,Min,Seconds
				Hour,Min=GetGameTime()
				Seconds=math.fmod(time(),60)
				TimeFormatted=string.format("%02d:%02d:%02d",Hour,Min,Seconds)

				Recount.InCombat=true
				Recount.InCombatT=Time
				Recount.InCombatF=TimeFormatted
			end
			--Want to return if we have someone in combat since don't want to get out yet
			return
		end
	end

	if Recount.InCombat then --We were in combat but no longer
		local Hour,Min,Seconds
		Hour,Min=GetGameTime()
		Seconds=math.fmod(time(),60)
		TimeFormatted=string.format("%02d:%02d:%02d",Hour,Min,Seconds)

		table.insert(Recount.db.char.CombatTimes,{Recount.InCombatT,Time,Recount.InCombatF,TimeFormatted})
		Recount.InCombat=false

		--Save current data as the last fight
		for name,v in pairs(Recount.db.char.combatants) do
			v.LastFightData=v.CurrentFightData
			v.CurrentFightData={}
			Recount:InitFightData(v.CurrentFightData)
		end
	end
end

function Recount:OnEnable()
	if Recount.db.char.combatants==nil then
		Recount.db.char.combatants={}
		Recount.db.char.CombatTimes={}
		Recount.db.char.version=DataVersion
	end
		
	if Recount.db.char.version~=DataVersion then
		Recount.db.char.combatants={}
		Recount.db.char.CombatTimes={}
		Recount.db.char.version=DataVersion
	end


	--Recount:Print("Registering Events")
	Parser:RegisterInfoType("Recount","hit",Recount_ParseDamageEvent)
	Parser:RegisterInfoType("Recount","miss",Recount_ParseMissEvent)
	Parser:RegisterInfoType("Recount","heal",Recount_ParseHealEvent)

	--Recount:RegisterEvent("RosterLib_RosterUpdated","RosterChange")
	Recount:CreateMainWindow()
	Recount:CreateDetailWindow()
	Recount:CreateGraphWindow()
	Recount:LoadMainWindowData(MainWindowModes)	
	Recount:ScheduleRepeatingEvent(Recount.TimeTick,1)

	Recount.TimeStep=1
	Recount.InCombat=false
end